import bpy

# Define the functions you want to add to the UI
def recalculate_normals_outside(self, context):
    for obj in bpy.context.selected_objects:
        if obj.type == 'MESH':
            bpy.context.view_layer.objects.active = obj
            bpy.ops.object.mode_set(mode='EDIT')
            bpy.ops.mesh.select_all(action='SELECT')
            bpy.ops.mesh.normals_make_consistent(inside=False)
            bpy.ops.object.mode_set(mode='OBJECT')

def enable_auto_smooth(self, context):
    bpy.ops.object.select_all(action='DESELECT')
    bpy.ops.object.select_by_type(type='MESH')
    
    for obj in bpy.context.selected_objects:
        if obj.data.use_auto_smooth == False:
            obj.data.use_auto_smooth = True

def duplicate_join_merge_by_distance(self, context):
    bpy.ops.object.select_all(action='DESELECT')
    bpy.ops.object.select_by_type(type='MESH')
    
    # Deselect active object
    bpy.context.selected_objects.remove(bpy.context.active_object)
    
    # Duplicate selected objects
    bpy.ops.object.duplicate(linked=False)
    
    # Join duplicated objects
    bpy.ops.object.join()
    
    # Enter Edit Mode and merge by distance
    bpy.ops.object.mode_set(mode='EDIT')
    bpy.ops.mesh.select_all(action='SELECT')
    bpy.ops.mesh.remove_doubles(threshold=0.001)  # You can adjust the threshold as needed
    bpy.ops.object.mode_set(mode='OBJECT')

# Function to calculate total statistics
def calculate_total_statistics():
    total_objects = len(bpy.context.selected_objects)
    total_vertices = sum(len(obj.data.vertices) for obj in bpy.context.selected_objects)
    total_edges = sum(len(obj.data.edges) for obj in bpy.context.selected_objects)
    total_triangles = sum(len(obj.data.polygons) for obj in bpy.context.selected_objects)
    return total_objects, total_vertices, total_edges, total_triangles

# Function to propagate shape keys for selected objects
def propagate_shape_keys(self, context):
    for obj in bpy.context.selected_objects:
        if obj.type == 'MESH':
            # Make sure the object has shape keys
            if obj.data.shape_keys:
                bpy.context.view_layer.objects.active = obj
                bpy.ops.object.mode_set(mode='EDIT')
                bpy.ops.mesh.select_all(action='SELECT')
                
                # Iterate through all shape keys and set their value to 0
                for shape_key in obj.data.shape_keys.key_blocks:
                    if shape_key != obj.data.shape_keys.reference_key:
                        shape_key.value = 0.0  # Set the value to 0
                
                bpy.ops.object.mode_set(mode='OBJECT')

# Function to blend from shape keys for selected objects
def blend_from_shape_keys(self, context):
    for obj in bpy.context.selected_objects:
        if obj.type == 'MESH':
            # Make sure the object has shape keys
            if obj.data.shape_keys:
                bpy.context.view_layer.objects.active = obj
                bpy.ops.object.mode_set(mode='EDIT')
                bpy.ops.mesh.select_all(action='SELECT')
                
                # Iterate through all shape keys and blend from them
                for shape_key in obj.data.shape_keys.key_blocks:
                    if shape_key != obj.data.shape_keys.reference_key:
                        shape_key.value = 0.0  # Set the value to 0 to blend from this shape key
                
                bpy.ops.object.mode_set(mode='OBJECT')

# Function to set normals from faces for selected objects
def set_normals_from_faces(self, context):
    for obj in bpy.context.selected_objects:
        if obj.type == 'MESH':
            bpy.context.view_layer.objects.active = obj
            bpy.ops.object.mode_set(mode='EDIT')
            bpy.ops.mesh.select_all(action='SELECT')
            
            # Recalculate normals from faces
            bpy.ops.mesh.normals_make_consistent(inside=False)
            
            bpy.ops.object.mode_set(mode='OBJECT')

# Create operators for the functions
class RecalculateNormalsOutsideOperator(bpy.types.Operator):
    bl_idname = "mesh.recalculate_normals_outside"
    bl_label = "Recalculate Normals (Outside)"
    
    def execute(self, context):
        recalculate_normals_outside(self, context)
        return {'FINISHED'}

class EnableAutoSmoothOperator(bpy.types.Operator):
    bl_idname = "mesh.enable_auto_smooth"
    bl_label = "Enable Auto Smooth"
    
    def execute(self, context):
        enable_auto_smooth(self, context)
        return {'FINISHED'}

class DuplicateJoinMergeByDistanceOperator(bpy.types.Operator):
    bl_idname = "mesh.duplicate_join_merge_by_distance"
    bl_label = "Duplicate, Join, Merge by Distance"
    
    def execute(self, context):
        duplicate_join_merge_by_distance(self, context)
        return {'FINISHED'}

# Create an operator for propagating shape keys
class PropagateShapeKeysOperator(bpy.types.Operator):
    bl_idname = "mesh.propagate_shape_keys"
    bl_label = "Propagate Shape Keys"
    
    def execute(self, context):
        propagate_shape_keys(self, context)
        return {'FINISHED'}

# Create an operator for blending from shape keys
class BlendFromShapeKeysOperator(bpy.types.Operator):
    bl_idname = "mesh.blend_from_shape_keys"
    bl_label = "Blend From Shape Keys"
    
    def execute(self, context):
        blend_from_shape_keys(self, context)
        return {'FINISHED'}

# Create an operator for setting normals from faces
class SetNormalsFromFacesOperator(bpy.types.Operator):
    bl_idname = "mesh.set_normals_from_faces"
    bl_label = "Set Normals from Faces"
    
    def execute(self, context):
        set_normals_from_faces(self, context)
        return {'FINISHED'}

# Create a new tab in the N-panel
class VIEW3D_PT_2KStreamlineTools(bpy.types.Panel):
    bl_label = "2K Streamline Tools"
    bl_idname = "VIEW3D_PT_2KStreamlineTools"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "2K Streamline"

    def draw(self, context):
        layout = self.layout

        # Create a list of operator names for sorting
        operator_names = [
            "mesh.blend_from_shape_keys",
            "mesh.duplicate_join_merge_by_distance",
            "mesh.enable_auto_smooth",
            "mesh.propagate_shape_keys",
            "mesh.recalculate_normals_outside",
            "mesh.set_normals_from_faces",
        ]

        # Sort the operator names alphabetically
        operator_names.sort()

        # Calculate total statistics
        total_objects, total_vertices, total_edges, total_triangles = calculate_total_statistics()

        # Display total statistics
        layout.label(text=f"Total Selected Objects: {total_objects}")
        layout.label(text=f"Total Vertices: {total_vertices}")
        layout.label(text=f"Total Edges: {total_edges}")
        layout.label(text=f"Total Triangles: {total_triangles}")

        # Add buttons for the operators in alphabetical order
        for operator_name in operator_names:
            layout.operator(operator_name)


# Register the operators and panel
def register():
    bpy.utils.register_class(VIEW3D_PT_2KStreamlineTools)
    bpy.utils.register_class(RecalculateNormalsOutsideOperator)
    bpy.utils.register_class(EnableAutoSmoothOperator)
    bpy.utils.register_class(DuplicateJoinMergeByDistanceOperator)
    bpy.utils.register_class(PropagateShapeKeysOperator)
    bpy.utils.register_class(BlendFromShapeKeysOperator)
    bpy.utils.register_class(SetNormalsFromFacesOperator)

def unregister():
    bpy.utils.unregister_class(VIEW3D_PT_2KStreamlineTools)
    bpy.utils.unregister_class(RecalculateNormalsOutsideOperator)
    bpy.utils.unregister_class(EnableAutoSmoothOperator)
    bpy.utils.unregister_class(DuplicateJoinMergeByDistanceOperator)
    bpy.utils.unregister_class(PropagateShapeKeysOperator)
    bpy.utils.unregister_class(BlendFromShapeKeysOperator)
    bpy.utils.unregister_class(SetNormalsFromFacesOperator)

if __name__ == "__main__":
    register()
